/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005 CAHA, Spain
! IDENT        extract_aper.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      C version of extract_aper.c
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  08.08.05 : Creation, SF.
------------------------------------------------------------------------------*/



#include <IFU_io.h>

#include "stats.h"
#include <stdlib.h>

#include <stdio.h> //For Standard I/O
#include <ctype.h> //For Type check
#include <string.h> //For String manipulation
#include <math.h> //For Math functions
#include <data_io.h> //For Lyon I/O data

#include <cpgplot.h> // PGPLOT


#define NB_SPEC 256
#define NPIX 1024





main(int argc, char *ARGV[])
{

  
    IMAGE2D image,trace,output;
    
    float *array,*tmp_array;
    char infile[80],trace_file[80],out_file[80];
    int spec_axis,i,k,j_min,j_max,j;
    float q;
    float y_shift,a,in_array,w,mod;
    float peak_y_max_new,range,flux,sum_w;

    char **argval, **arglabel;
    long npix[2];
    double start[2];
    double step[2];
    char tmp_char[80];
    float aperture,fwhm,delta,sub,aperture2;

    float *a_out,*a_out_new;

    int ny,n_conver;
    int *conver;
    float *we;
    int niter,niter_max;
    int color=7;
    float f;
    float c1,c2;

    int plot;
    int n_adj,kk;

    float *x,*y,*yy,*ymod;
    float fmin,fmax;
    int nplot=1;
    int iplot=0;
    float xp[2],yp[2];


    if (argc<8) {
	printf("USE: extract_aper RAW.fits Spectral_axis[0/1] TRACE.fits Aperture OUTPUT.fits FWHM plot [SHIFT] [2ndAperture=Aperture] [n_adjacents=1] [nplot=1] [niter=1]\n");
	return(0);
    }
    
 set_user_dataformat();
//    init_session(ARGV,argc,&arglabel,&argval);

    strcpy(infile,ARGV[1]);
    spec_axis=atoi(ARGV[2]);
    strcpy(trace_file,ARGV[3]);
    aperture=atof(ARGV[4]);
    strcpy(out_file,ARGV[5]);
    fwhm=atof(ARGV[6]);
//    delta=atof(ARGV[7]);
//    niter_max=atoi(ARGV[7]);
    plot=atoi(ARGV[7]);

    y_shift=0;
    if (argc==9) {
	y_shift=atof(ARGV[8]);
    }
    aperture2=aperture;
    if (argc==10) {
	y_shift=atof(ARGV[8]);
	aperture2=atof(ARGV[9]);
    }
    n_adj=1;
    if (argc==11) {
	y_shift=atof(ARGV[8]);
	aperture2=atof(ARGV[9]);
	n_adj=atoi(ARGV[10]);
    }
    if (argc==12) {
	y_shift=atof(ARGV[8]);
	aperture2=atof(ARGV[9]);
	n_adj=atoi(ARGV[10]);
	nplot=atoi(ARGV[11]);
    }

    niter_max=1;
    if (argc==13) {
	y_shift=atof(ARGV[8]);
	aperture2=atof(ARGV[9]);
	n_adj=atoi(ARGV[10]);
	nplot=atoi(ARGV[11]);
	niter_max=atoi(ARGV[12]);
    }

    c1=(exp(-0.5*(pow((aperture)/(fwhm/2.345),2)))/(2.5066*(fwhm/2.345)));

    printf("Reading file infile '%s'",infile);
    open_frame(&image,infile,"I");
    printf("DONE\n");
    printf("Reading file trace_file '%s'",trace_file);
    open_frame(&trace,trace_file,"I");
    printf("DONE\n");
    printf("Creating the output file '%s'",out_file);

    if (spec_axis==0) {
	if (trace.nx!=image.nx) {
	    printf("The X-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.nx);
	    return(0);
	}    
	ny=image.ny;
    } else {
	if (trace.nx!=image.ny) {
	    printf("The Y-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.nx);
	    return(0);
	}    
	ny=image.nx;
    }
    a_out=(float *)malloc((trace.nx*trace.ny)*sizeof(float));
    a_out_new=(float *)malloc((trace.nx*trace.ny)*sizeof(float));
    we=(float *)malloc((ny*trace.ny)*sizeof(float));
    conver=(int *)malloc((trace.ny)*sizeof(int));

    npix[0]=image.nx;
    npix[1]=trace.ny;
    start[0]=0.0;
    start[1]=0.0;
    step[0]=1.0;
    step[1]=1.0;

//    printf("DONE\n");

   


    printf("Extracting...\n");


    for (i=0;i<trace.nx;i++) {
	for (k=0;k<trace.ny;k++) {
	    a=0.5*aperture;
	    peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
	    range=(int)(1.5*aperture);
	    j_min=(int)(peak_y_max_new-1.5*aperture);
	    j_max=(int)(peak_y_max_new+1.5*aperture);
	    if (j_min<0) {
		j_min=0;
	    }
	    if (j_max>=image.ny) {
		j_max=image.ny-1;
	    }
//	    printf("%d %d %d,%d %f %f\n",i,k,j_min,j_max,peak_y_max_new,aperture2);
	    flux=0;
	    sum_w=0;
	    q=peak_y_max_new;
	    for (j=j_min;j<j_max;j++) {
		if (spec_axis==0) {
		    in_array=RD_frame(&image,i,j);
		} else {
		    in_array=RD_frame(&image,j,i);
		}
		w=0;
		if (((q-a)<=(j-0.5))&&((q+a)>=(j+0.5))) {
		    w=1;
		}
		if (((q-a)<=(j-0.5))&&((q+a)<(j+0.5))) {
		    w=q+a-(j-0.5);
		}
		if (((q-a)>(j-0.5))&&((q+a)>=(j+0.5))) {
		    w=j+0.5-(q-a);
		}
		if (w<0) {
		    w=0;
		}
		if (w>1) {
		    w=0;
		}
		sum_w=sum_w+w;
		flux=flux+w*in_array;
	    }
	    a_out[i+trace.nx*k]=flux;
	    a_out_new[i+trace.nx*k]=flux;
	    for (j=0;j<ny;j++) {
		we[j+k*ny]=(exp(-0.5*(pow((j*1.0-peak_y_max_new)/(fwhm/2.345),2)))/(2.5066*(fwhm/2.345)));
	    }
//	    conver[k]=0;
//	    n_conver=0;
	}
	
//	niter=0;
//    }


//    printf("Cleaning cross-talk...\n");
//	for (i=0;i<trace.nx;i++) {
	niter=0;
	do {
	    if (niter==(niter_max-1)) {
	        a=0.5*aperture2;
	    } else {
		a=0.5*aperture;
	    }
	    for (k=0;k<trace.ny;k++) {
		peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
		range=(int)(1.5*aperture2);
		j_min=(int)(peak_y_max_new-1.5*aperture2);
		j_max=(int)(peak_y_max_new+1.5*aperture2);
//	    printf("%d %d %d,%d %f %f\n",i,k,j_min,j_max,peak_y_max_new,aperture2);
		if (j_min<0) {
		    j_min=0;
		}
		if (j_max>=image.ny) {
		    j_max=image.ny-1;
		}
		x = (float *)malloc((j_max-j_min)*sizeof(float));  
		y = (float *)malloc((j_max-j_min)*sizeof(float));  
		yy = (float *)malloc((j_max-j_min)*sizeof(float));  
		ymod = (float *)malloc((j_max-j_min)*sizeof(float));  
		fmin=1e12;
		fmax=-1e12;
		flux=0;
		sum_w=0;
		q=peak_y_max_new;
		for (j=j_min;j<j_max;j++) {
		    if (spec_axis==0) {
			in_array=RD_frame(&image,i,j);
		    } else {
			in_array=RD_frame(&image,j,i);
		    }
		    x[j-j_min]=1.0*j;
		    y[j-j_min]=in_array;
		    if (fmin>y[j-j_min]) {
			fmin=y[j-j_min];
		    }
		    if (fmax<y[j-j_min]) {
		    fmax=y[j-j_min];
		    }	    
		    
		    /*
		     * We clean now the contribution from the two
		     * nearest fibers adjacent fibers
		     */
		    mod=0;
		    for (kk=1;kk<=n_adj;kk++) {
			if ((k-kk)>-1) {
			    in_array=in_array-we[j+(k-kk)*ny]*a_out[i+trace.nx*(k-kk)];
			    mod=mod+we[j+(k-kk)*ny]*a_out[i+trace.nx*(k-kk)];
			}
			if ((k+kk)<trace.ny) {
			    in_array=in_array-we[j+(k+kk)*ny]*a_out[i+trace.nx*(k+kk)];
			    mod=mod+we[j+(k+kk)*ny]*a_out[i+trace.nx*(k+kk)];
			}		    		    
		    }
		    mod=mod+we[j+k*ny]*a_out[i+trace.nx*k];
		    ymod[j-j_min]=mod;
		    yy[j-j_min]=in_array;
		    if (fmin>yy[j-j_min]) {
			fmin=yy[j-j_min];
		    }
		    
		    w=0;
		    if (((q-a)<=(j-0.5))&&((q+a)>=(j+0.5))) {
			w=1;
		    }
		    if (((q-a)<=(j-0.5))&&((q+a)<(j+0.5))) {
			w=q+a-(j-0.5);
		    }
		    if (((q-a)>(j-0.5))&&((q+a)>=(j+0.5))) {
			w=j+0.5-(q-a);
		    }
		    if (w<0) {
			w=0;
		    }
		    if (w>1) {
			w=0;
		    }
		    sum_w=sum_w+w;
//		printf("%f %f %f %f %f\n",w,in_array,flux,q,a);
		    flux=flux+w*in_array;
		}


	    if (plot!=0) {
		if (niter==(niter_max-1)) {
		if (fmin!=fmax) {
//		    if (k==10*nearbyint(k/10)) {
			if (iplot==0) {
			    if (plot==1) {
				cpgopen("/xs");
			    } else {
				cpgopen("extract_aper_CT.ps/CPS");
			    }
			    cpgask(0);
			    cpgscf(2);             
			    cpgslw(2);
			    cpgsch(1.3);
			    cpgsubp(nplot,nplot);
//			cpgenv(j_min-aperture2,j_max+aperture2,fmin-2,fmax,0,0);
			    cpgenv(j_min,j_max,fmin-0.1,fmax+0.1,0,0);
//			    cpgenv(j_min,j_max,-2,fmax,0,0);
			iplot++;
			} else {
			    cpgenv(j_min,j_max,fmin-0.1,fmax+0.1,0,0);
//			    cpgenv(j_min,j_max,-2,fmax,0,0);
			}
			cpgsci(1);
			cpgline(j_max-j_min,x,y);    
			cpgsci(3);
			cpgsls(3);
			cpgline(j_max-j_min,x,ymod);    
			
//			cpgsci(color);
			cpgsci(2);
			cpgsls(1);
			cpgline(j_max-j_min,x,yy);    
			cpgsci(4);
			xp[0]=peak_y_max_new-a;
			xp[1]=peak_y_max_new-a;
			yp[0]=0.2*fmax;
			yp[1]=fmin;
			cpgline(2,xp,yp);    
			xp[0]=peak_y_max_new+a;
			xp[1]=peak_y_max_new+a;
			yp[0]=0.2*fmax;
			yp[1]=fmin;
			cpgline(2,xp,yp);    
			cpgsci(1);		
			if (plot!=1) {
			    iplot=0;
			    cpgclos();
			    cpgend;	
			}
		    }
		}
//		}
	    }
		
	    free(x);
	    free(y);
	    free(yy);
	    free(ymod);
//	    if (flux<a_out_new[i+trace.nx*k]) {
		a_out_new[i+trace.nx*k]=flux;
//	    }
	    }	    
	    color=color++;
	    if (color>15) {
		color=7;
	    }
	    niter++;
	    for (k=0;k<trace.ny;k++) {
		a_out[i+trace.nx*k]=a_out_new[i+trace.nx*k];
	    }

	} while (niter<niter_max);
	

	
	if (i==20*nearbyint(i/20)) {
	    printf("%d,%d\n",i,image.nx);
	}
    }

	    if ((plot==1)&&(iplot!=0)) {
//		iplot=0;
		cpgclos();
		cpgend;	
	    }






/*
  Saving the results
*/
    create_frame(&output,out_file,npix,start,step,FLOAT,"EXTRACTED",NULL);
    for (i=0;i<trace.nx;i++) {
	for (k=0;k<trace.ny;k++) {    
	    WR_frame(&output,i,k,a_out_new[i+trace.nx*k]);
//	    WR_frame(&output,i,k,a_out[i+trace.nx*k]);
	}
    }
    close_frame(&output);    
    printf("DONE\n");
//    exit_session(0);
    return(0);	
}



