/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005 CAHA, Spain
! IDENT        extract_aper.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      C version of extract_aper.c
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  08.08.05 : Creation, SF.
------------------------------------------------------------------------------*/

#include <IFU_io.h>

#include "stats.h"
#include <stdlib.h>

#include "Euro3D.h";

#define NB_SPEC 256
#define NPIX 1024

#include <gl_lsq.h> 

#include <cpgplot.h> // PGPLOT


main(int argc, char *ARGV[])
{

  
    IMAGE2D image,trace,output,varput;
    
    float *array,*tmp_array;
    char infile[80],trace_file[80],out_file[80],var_file[80];
    int spec_axis,i,k,j_min,j_max,j;
    float q;
    float y_shift,a,in_array,w;
    float peak_y_max_new,range,flux,sum_w;
    float peak_y_max_0,peak_y_max_1;
    float flux_near;
    float flux0,flux1;
    float *flux_array;
    float flux_old;


    char **argval, **arglabel;
    long npix[2];
    double start[2];
    double step[2];
    char tmp_char[80];
    float aperture;
    int J_max;
    float FWHM;
    int plot;
    float NORM;
    float A;
    float G,sum;
    int kk;
    int kkk;
    int k2;
    int jj;
    float *y_mod;
    float *y_res;
    float res_mean,res_sigma;

    float cut_sum;
    float *cut;
    float *cut_new;
    float *noise_new;
    float *xcut;
    float min,max,xmin,xmax;
    char command;
    
    float C[4],s_a[4],d_a[4];
    float chisq,chisq_lim;
    int ia[4];
    float y_tmp;
    float ci[4],si[4],ai[4];


    int nad;

    float *flux_ad;
    float *peak_ad;

    if (argc<9) {
	printf("USE: extract_gauss_simple RAW.fits Spectral_axis[0/1] TRACE.fits Aperture OUTPUT.fits VARIANCE.fits WIDTH PLOT [NAD] [SHIFT]\n");
	return(0);
    }
     set_user_dataformat();
//    init_session(ARGV,argc,&arglabel,&argval);

    strcpy(infile,ARGV[1]);
    spec_axis=atoi(ARGV[2]);
    strcpy(trace_file,ARGV[3]);
    aperture=atof(ARGV[4]);
    strcpy(out_file,ARGV[5]);
    strcpy(var_file,ARGV[6]);
    FWHM=atof(ARGV[7]);
    plot=atoi(ARGV[8]);
    
    A=0.5/pow((fabs(FWHM)/2.345),2);
    NORM=sqrt(3.1416/A);

    nad=1;
    if (argc>9) {
	nad=atoi(ARGV[9]);
    }
//    printf("%d %d\n",argc,nad);

    y_shift=0;
    if (argc==11) {
	y_shift=atof(ARGV[10]);
    }

    printf("Reading file infile '%s'",infile);
    open_frame(&image,infile,"I");
    printf("DONE\n");
    printf("Reading file trace_file '%s'",trace_file);
    open_frame(&trace,trace_file,"I");
    printf("DONE\n");
    printf("Creating the output file '%s'",out_file);

    if (spec_axis==0) {
	if (trace.nx!=image.nx) {
	    printf("The X-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.nx);
	    return(0);
	}    
    } else {
	if (trace.nx!=image.ny) {
	    printf("The Y-dimension of the trace file (trace_file), nx_tr\n");
	    printf("does not match that of the original image, nx\n");
	    printf("%d!=%d",trace.nx,image.ny);
	    return(0);
	}    
    }
    if (spec_axis==0) {
	npix[0]=image.nx;
	npix[1]=trace.ny;
	J_max=image.ny;
    } else {
	npix[0]=image.ny;
	npix[1]=trace.ny;
	J_max=image.nx;
    }
    start[0]=0.0;
    start[1]=0.0;
    step[0]=1.0;
    step[1]=1.0;
    create_frame(&output,out_file,npix,start,step,FLOAT,"EXTRACTED",NULL);
    create_frame(&varput,var_file,npix,start,step,FLOAT,"EXTRACTED",NULL);
    printf("DONE\n");

    flux_array = (float *) malloc( (npix[0]*npix[1])*sizeof(float));    

    flux_ad = (float *) malloc( (2*nad)*sizeof(float));    
    peak_ad = (float *) malloc( (2*nad)*sizeof(float));    


    a=0.5*aperture;
    printf("Extracting...");


    for (i=0;i<trace.nx;i++) {


	if (i==50*nearbyint(i/50)) {
	    printf("%d,%d\n",i,npix[0]);
	}


	for (k=0;k<trace.ny;k++) {
	    peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
	    if (peak_y_max_new<0) {
		peak_y_max_new=0;
	    }
	    if (peak_y_max_new>J_max) {
		peak_y_max_new=J_max;
	    }

	    range=(1.5*aperture);
	    j_min=round(peak_y_max_new-range);
	    j_max=round(peak_y_max_new+range);
	    if (j_min<0) {
		j_min=0;
	    }
	    if (j_max>=J_max) {
		j_max=J_max-1;
	    }

	    cut = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    cut_new = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    y_mod = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    noise_new = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    xcut = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    min = 1e12;
	    max = -1e12;
		
	    cut_sum=0;
	    for (kkk=0;kkk<nad;kkk++) {
		if ((k-kkk)>0) {
		    peak_ad[kkk]=RD_frame(&trace,i,k-kkk-1)+y_shift;			
		    flux_ad[kkk]=1;		
		} else {
		    peak_ad[kkk]=peak_y_max_new;
		    flux_ad[kkk]=0;		    
		}
	    }
	    for (kkk=0;kkk<nad;kkk++) {
		if ((k+kkk)<(trace.ny-1)) {
		    peak_ad[nad+kkk]=RD_frame(&trace,i,k+kkk+1)+y_shift;	
		    flux_ad[nad+kkk]=1;
		} else {
		    peak_ad[nad+kkk]=peak_y_max_new;
		    flux_ad[nad+kkk]=0;
		    
		}
	    }

/*
	    for (kkk=0;kkk<2*nad;kkk++) {
		printf("peak=%f, k=%d peak_ad=%f\n",peak_y_max_new,kkk,peak_ad[kkk]);
	    }
*/
	    sum=0;		
	    kk=0;
	    
	    for (j=j_min;j<j_max;j++) {
		if (spec_axis==0) {
		    in_array=RD_frame(&image,i,j);
		} else {
		    in_array=RD_frame(&image,j,i);
		}
		
		if (in_array>max) {
		    max=in_array;
		}
		if (in_array<min) {
		    min=in_array;
		}
		
		
		cut[kk]=in_array;
		if (cut[kk]==(cut[kk]/2)) {
		    cut[kk]=0;
		}
		cut_new[kk]=cut[kk];
		cut_sum=cut_sum+in_array;
		xcut[kk]=j*1.000;
		
		kk++;

	    }
	


	    
	    C[0]=flux/NORM+10;
	    C[1]=peak_y_max_new;
	    C[2]=FWHM/2.345;
	    C[3]=0;
	    d_a[0]=3*flux+10;
	    d_a[1]=1;
	    d_a[2]=1;
	    d_a[3]=1;
	    ia[0]=1;
	    ia[1]=0;
	    ia[2]=0;
	    ia[3]=0;
	    chisq_lim=1;

	    /*
	     * We the adjacent fibers
	     */
	    for (jj=0;jj<2*nad;jj++) {
		si[0]=FWHM/2.345;
		ci[0]=peak_ad[jj];
		ai[0]=C[0];

		if (cut_sum!=0.0) {
		    multi_gaussfit(xcut,cut,kk,ai,1,ci,si);
		} else {
		    ai[0]=0.0;
		}
		if (ai[0]<0) {
		    ai[0]=0;
		}
		C[0]=ai[0];
		C[1]=peak_ad[jj];
		C[2]=FWHM/2.345;
		C[3]=0;
		for (kkk=0;kkk<kk;kkk++) {
		    gl_fit_gauss(xcut[kkk],C,&y_tmp);
		    if (xcut[kkk]>=peak_y_max_0) {
			cut_new[kkk]=cut_new[kkk]-flux_ad[jj]*y_tmp;
		    } else {
			cut_new[kkk]=0;
		    }
		}
	    }

	    /*
	     * Then we fit the data
	     */
	    C[0]=ai[0];
	    C[1]=peak_y_max_new;
	    C[2]=FWHM/2.345;
	    C[3]=0;

	    
	    si[0]=C[2];
	    ci[0]=peak_y_max_new;
	    ai[0]=C[0];

	    if (cut_sum!=0.0) {
		multi_gaussfit(xcut,cut_new,kk,ai,1,ci,si);
	    } else {
		ai[0]=0.0;
	    }
	    if (ai[0]<0) {
		ai[0]=0;
	    }

	    flux=ai[0]*NORM;
	    if ((flux == (flux/2))&&(flux!=0)) {
		flux=65536;
	    }


	    C[0]=ai[0];
	    for (kkk=0;kkk<kk;kkk++) {
		gl_fit_gauss(xcut[kkk],C,&y_tmp);
		y_mod[kkk]=y_tmp;
	    }
	    C[0]=C[0]*NORM;


	    
	    if (plot==2) {
		if (i==50*nearbyint(i/50)) {
		    cpgopen("/xs");
		cpgask(0); 
		cpgscf(2);             //  Set character font
		cpgslw(2);             //  Set line width
		cpgsch(1.4);           //  Set character height
		xmin=xcut[0];
		xmax=xcut[kk-1];
		cpgenv(xmin*1.0,xmax*1.0,-0.2*max,max,0,0);
		cpglab("Y-axis","Counts","");
		cpgsci(1);
		cpgpt(kk,xcut,cut,3);
		cpgsls(2);
		cpgline(kk,xcut,cut);
		cpgsls(1);
		cpgsci(2);
		cpgpt(kk,xcut,cut_new,12);
		cpgsls(2);
		cpgline(kk,xcut,cut_new);
		cpgsci(3);
		cpgsls(1);
		cpgline(kk,xcut,y_mod);
		cpgsci(1);
		cpgsls(1);
		cpgclos();
		cpgend();
//		printf("Press Enter\n");
//		getc(stdin);
		}
	    }

	    free(cut);
	    free(cut_new);
	    free(noise_new);
	    free(xcut);
	    free(y_mod);
	    flux_old=flux;

//	    printf("%d,%d %f\n",i,k,flux);
//	    WR_frame(&output,i,k,flux_old);

	    flux_array[i+k*npix[0]]=flux;
//	    WR_frame(&output,i,k,flux);
	}

/*
 * Second iteration
 */

	for (k=0;k<trace.ny;k++) {
	    peak_y_max_new=RD_frame(&trace,i,k)+y_shift;	
	    if (peak_y_max_new<0) {
		peak_y_max_new=0;
	    }
	    if (peak_y_max_new>J_max) {
		peak_y_max_new=J_max;
	    }

	    range=(1.5*aperture);
	    j_min=round(peak_y_max_new-range);
	    j_max=round(peak_y_max_new+range);
	    if (j_min<0) {
		    j_min=0;
	    }
	    if (j_max>=J_max) {
		j_max=J_max-1;
	    }

	    cut = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    cut_new = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    y_mod = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    y_res = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    noise_new = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    xcut = (float *) malloc( (j_max-j_min+1)*sizeof(float));
	    min = 1e12;
	    max = -1e12;
		
	    cut_sum=0;
	    for (kkk=0;kkk<nad;kkk++) {
		if ((k-kkk)>0) {
		    peak_ad[kkk]=RD_frame(&trace,i,k-kkk-1)+y_shift;			
		    flux_ad[kkk]=1;		
		} else {
		    peak_ad[kkk]=peak_y_max_new;
		    flux_ad[kkk]=0;		    
		}
	    }
	    for (kkk=0;kkk<nad;kkk++) {
		if ((k+kkk)<(trace.ny-1)) {
		    peak_ad[nad+kkk]=RD_frame(&trace,i,k+kkk+1)+y_shift;	
		    flux_ad[nad+kkk]=1;
		} else {
		    peak_ad[nad+kkk]=peak_y_max_new;
		    flux_ad[nad+kkk]=0;
		    
		}
	    }

/*
	    for (kkk=0;kkk<2*nad;kkk++) {
		printf("peak=%f, k=%d peak_ad=%f\n",peak_y_max_new,kkk,peak_ad[kkk]);
	    }
*/
	    sum=0;		
	    kk=0;
	    
	    for (j=j_min;j<j_max;j++) {
		if (spec_axis==0) {
		    in_array=RD_frame(&image,i,j);
		} else {
		    in_array=RD_frame(&image,j,i);
		}
		
		if (in_array>max) {
		    max=in_array;
		}
		if (in_array<min) {
		    min=in_array;
		}
		
		
		cut[kk]=in_array;
		cut_new[kk]=in_array;
		cut_sum=cut_sum+in_array;
		xcut[kk]=j*1.000;
		
		kk++;

	    }
	


	    
	    C[0]=flux/NORM+10;
	    C[1]=peak_y_max_new;
	    C[2]=FWHM/2.345;
	    C[3]=0;
	    d_a[0]=3*flux+10;
	    d_a[1]=1;
	    d_a[2]=1;
	    d_a[3]=1;
	    ia[0]=1;
	    ia[1]=0;
	    ia[2]=0;
	    ia[3]=0;
	    chisq_lim=1;

	    /*
	     * We the adjacent fibers
	     */
	    for (jj=0;jj<2*nad;jj++) {
		if (jj<nad) {
		    if ((k-jj-1)>0) {
			C[0]=flux_array[i+(k-jj-1)*npix[0]]/NORM;
		    } else {
			C[0]=0.0;
		    }
		} else {
		    if ((k+(jj-nad)+1)<trace.ny) {
			C[0]=flux_array[i+(k+(jj-nad)+1)*npix[0]]/NORM;
		    } else {
			C[0]=0.0;
		    }
		}
		C[1]=peak_ad[jj];
		C[2]=FWHM/2.345;
		C[3]=0;
		for (kkk=0;kkk<kk;kkk++) {
		    gl_fit_gauss(xcut[kkk],C,&y_tmp);
		    if (xcut[kkk]>=peak_y_max_0) {
			cut_new[kkk]=cut_new[kkk]-flux_ad[jj]*y_tmp;
		    } else {
			cut_new[kkk]=0;
		    }
		}
	    }

	    /*
	     * Then we fit the data
	     */
	    C[0]=ai[0];
	    C[1]=peak_y_max_new;
	    C[2]=FWHM/2.345;
	    C[3]=0;

	    
	    si[0]=C[2];
	    ci[0]=peak_y_max_new;
	    ai[0]=C[0];

	    if (cut_sum!=0.0) {
		multi_gaussfit(xcut,cut_new,kk,ai,1,ci,si);
	    } else {
		ai[0]=0.0;
	    }
	    if (ai[0]<0) {
		ai[0]=0;
	    }

	    flux=ai[0]*NORM;
	    if ((flux == (flux/2))&&(flux!=0)) {
		flux=65536;
	    }


	    C[0]=ai[0];
	    k2=0;
	    for (kkk=0;kkk<kk;kkk++) {
		gl_fit_gauss(xcut[kkk],C,&y_tmp);
		y_mod[kkk]=y_tmp;
		if (fabs(xcut[kkk]-peak_y_max_new)<FWHM) {
		    y_res[k2]=cut_new[kkk]-y_mod[kkk];
		    k2++;
		}
//		printf("%d %f %f %f %f\n",kkk,cut[kkk],cut_new[kkk],y_mod[kkk],y_res[kkk]);
	    }
	    C[0]=C[0]*NORM;
	    if (k2>0) {
		res_mean=mean(y_res,k2);
		res_sigma=sigma(y_res,k2,res_mean);
		res_sigma=pow(res_sigma,2);
//	    printf("RES=%f\n",res_sigma);
	    } else {
		res_sigma=0;
	    }
    
	    if (plot==1) {
		if (i==50*nearbyint(i/50)) {
		    cpgopen("/xs");
		cpgask(0); 
		cpgscf(2);             //  Set character font
		cpgslw(2);             //  Set line width
		cpgsch(1.4);           //  Set character height
		xmin=xcut[0];
		xmax=xcut[kk-1];
		cpgenv(xmin*1.0,xmax*1.0,-0.2*max,max,0,0);
		cpglab("Y-axis","Counts","");
		cpgsci(1);
		cpgpt(kk,xcut,cut,3);
		cpgsls(2);
		cpgline(kk,xcut,cut);
		cpgsls(1);
		cpgsci(2);
		cpgpt(kk,xcut,cut_new,12);
		cpgsls(2);
		cpgline(kk,xcut,cut_new);
		cpgsci(3);
		cpgsls(1);
		cpgline(kk,xcut,y_mod);
		cpgsci(1);
		cpgsls(1);
		cpgclos();
		cpgend();
//		printf("Press Enter\n");
//		getc(stdin);
		}
	    }

	    free(cut);
	    free(cut_new);
	    free(noise_new);
	    free(xcut);
	    free(y_mod);
	    free(y_res);

	    flux_old=flux;

//	    printf("%d,%d %f\n",i,k,flux);
//	    WR_frame(&output,i,k,flux_old);

//	    flux_array[i+k*npix[0]]=flux;
	    WR_frame(&output,i,k,flux);
	    WR_frame(&varput,i,k,res_sigma);
	}



	

    }



//    printf("closing\n");

    close_frame(&output);    
    close_frame(&trace);    
    close_frame(&image);    
    close_frame(&varput);    
    printf("DONE\n");

    return(0);	
}



